import subprocess
import os

from .core import run
from .file_mgr import truncated_files, rpath, clean_dir, copy_file

# parse the file for errors
# TODO: this is garbage, fix it
def parse_errors(filename):
    dct = {'errors':[],'warnings':[],'fatal':[]}
    # currently doesn't catch warnings
    with open(rpath(filename + ".log")) as f:
        append = False
        temp = ""
        for l in f:
            if l.startswith("! "):
                dct['fatal'] += [l]
            if l.startswith("./" + filename + ".tex:"):
                temp = l
                append = True
            if append:
                temp += l
            if l.startswith("l."):
                temp += l
                append = False
                dct['errors'] += [temp]
    return dct

# check if a dct has any non-empty lists
def is_empty(dct):
    for key in dct.keys():
        if dct[key]:
            return False
    return True

# run the test
def test():
    # clean log/test directory
    clean_dir("log")
    clean_dir("test")

    # iterate over possible template names
    for tm in truncated_files("share","templates"):
        # build the template in "test"
        run(rpath("test","test.tex"), tm)

        # compile the template
        lmk_args = [
                'latexmk',
                '-pdf',
                '-interaction=nonstopmode',
                '-outdir={}'.format(rpath("test")),
                rpath("test","test.tex")]
        p2 = subprocess.Popen(lmk_args, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL) # TODO: not good to suppress errors
        p2.wait()

        # parse for errors, print errors if they exist
        e = parse_errors("test/test")
        if is_empty(e):
            print("No errors in template '{}'".format(tm))
        else:
            # if there are errors, we copy the relevant files to the log folder
            print("Errors in template '{}'; .tex file can be found in the log folder at '{}'.".format(tm,rpath("log")))
            copy_file(rpath("test","test.tex"),rpath("log",tm + ".tex"))
            copy_file(rpath("test","test.log"),rpath("log",tm + ".log"))

        # clean up
        clean_dir("test")



"""
TexNew

Automatic LaTeX template management.

:copyright: (c) 2019 by Alex Rutar
:license: MIT, see LICENSE for more details.
"""

__version__ = "1.0"

from texnew.test import parse_errors
from texnew.update import update
from texnew.core import run
from texnew.test import test



import os
import re
from os.path import expanduser

from . import __path__

# check for file version
def get_version(filename):
    with open(filename,'r') as f:
        st = f.read()
    pat = re.compile(r"% version \((.*)\)")
    res = pat.search(st)
    if res:
        return res.group(1)
    else:
        return "0.1"

# create a new block divider
def get_div(name):
    return ("% " + name + " ").ljust(80, "-") + "\n"

# get the relative path, from this script
def rpath(*rel_path):
    return os.path.join(expanduser("~"),".texnew",*rel_path)

# remove the file endings
def truncated_files(*rel_path):
    return ["".join(s.split(".")[:-1]) for s in os.listdir(rpath(*rel_path))]

# clean the directory at a relative path
def clean_dir(*rel_path):
    for fl in os.listdir(rpath(*rel_path)):
        if not fl.startswith("."):
            os.remove(rpath(*rel_path,fl))

# read file at a relative path
def filestring(*rel_path):
    with open(rpath(*rel_path)) as f:
        out = f.read()
    return out

# copy a file to the target; only appends, does not overwrite
def copy_file(src,trg):
    with open(src,'r') as f, open(trg,'a+') as output:
        for l in f:
            output.write(l)

# get an available name, inserting 'ad' if necessary
def get_name(name,ad):
    if "." in name:
        base = "".join(name.split(".")[:-1])
        ftype = name.split(".")[-1]
    else:
        base = name
        ftype = ""
    for t in [""] + ["_"+str(x) for x in range(1000)]:
        attempt = base + ad + t + "." + ftype
        if not os.path.exists(attempt):
            return attempt

# check if a string is a divider (with any name)
def is_div(st):
    dv = get_div("")
    return len(st) == len(dv) and st.startswith(dv[:2]) and st.endswith(dv[-3:])

# separate a block named div_name
def sep_block(lst, div_name):
    start = get_div(div_name)
    read = False
    id1 = None
    id2 = None
    for e,l in enumerate(lst):
        if read and is_div(l):
            id2 = e
            break
        if l == start:
            id1 = e+1
            read = True
    if id1 is None:
        return []
    if id2 is None:
        return lst[id1:]
    return lst[id1:id2]



import yaml
import os
import re

from . import __version__
from .file_mgr import filestring, truncated_files, rpath, get_div

# print a divider to the specified output
def write_div(out, name):
    out.write("\n" + get_div(name))

# creates a matching regex for file substitution
def repl_match(name):
    if name == "any":
        return r"<\+.*\+>"
    else:
        return r"<\+" + str(name) + r"\+>"

# the main file-building function
def run_output(target,template_type,data,user_info,user_macros):
    tex_doctype = re.sub(repl_match("doctype"), data['doctype'], filestring("share","defaults","doctype.tex"))
    tex_packages = filestring("share","defaults","packages.tex")
    tex_macros = filestring("share","defaults","macros.tex")
    tex_formatting = filestring("share","formatting",data['formatting'] + '.tex')
    for k in user_info.keys():
        tex_formatting = re.sub(repl_match(k), str(user_info[k]), tex_formatting)

    with open(target,"a+") as output:
        output.write("% Template created by texnew (author: Alex Rutar); info can be found at 'https://github.com/alexrutar/texnew'.\n")
        output.write("% version ({})\n".format(__version__))

        # create doctype
        write_div(output, "doctype")
        output.write(tex_doctype)

        # add default packates
        write_div(output, "packages")
        output.write(tex_packages)

        # add included macros
        write_div(output, "default macros")
        output.write(tex_macros)
        for name in data['macros']:
            write_div(output, name+" macros")
            output.write(filestring("share","macros",name + ".tex"))


        # add space for user macros
        write_div(output, "file-specific macros")
        if 'macros' in user_macros.keys():
            for l in user_macros['macros']:
                output.write(l)
        else:
            output.write("% REPLACE\n")

        # add formatting file
        write_div(output, "formatting")
        output.write(tex_formatting)

        # check for contents in user_macros to fill document
        write_div(output, "document start")
        if 'contents' in user_macros.keys():
            for l in user_macros['contents']:
                output.write(l)
        else:
            output.write("\nREPLACE\n")
            output.write("\\end{document}"+"\n")

# return yaml information from a relative path
def load_yaml(*rel_path):
    with open(rpath(*rel_path),'r') as source:
        return yaml.load(source)

# load template information
def get_data(template_type):
    data = []
    try:
        data = load_yaml("share","templates",template_type + ".yaml")
    except FileNotFoundError:
        print("The template \"{}\" does not exist! The possible template names are:\n".format(template_type)+ "\t".join(truncated_files("share","templates")))
    return data

# essentially a wrapper for run_output
def run(target, template_type, user_macros={}):
    if os.path.exists(target):
        print("Error: The file \"{}\" already exists. Please choose another filename.".format(target))
    else:
        try:
            user_info = load_yaml("user_private.yaml")
        except FileNotFoundError:
            try:
                user_info = load_yaml("user.yaml")
            except FileNotFoundError:
                user_info = {}
                print("Warning: user info file could not be found at '{}/user.yaml' or at '{}/user_private.yaml'."/format("share","share"))
        data = get_data(template_type)
        if data:
            run_output(target,template_type,data,user_info,user_macros)
